@@ -45,6 +45,7 @@ pub(crate) struct HiArgs {
     context: ContextMode,
     context_separator: ContextSeparator,
     crlf: bool,
+    cwd: PathBuf,
     dfa_size_limit: Option<usize>,
     encoding: EncodingMode,
     engine: EngineChoice,
@@ -262,6 +263,7 @@ impl HiArgs {
             context: low.context,
             context_separator: low.context_separator,
             crlf: low.crlf,
+            cwd: state.cwd,
             dfa_size_limit: low.dfa_size_limit,
             encoding: low.encoding,
             engine: low.engine,
@@ -484,9 +486,9 @@ impl HiArgs {
             if self.crlf {
                 builder.crlf(true);
             }
-            // We don't need to set this in multiline mode since mulitline
+            // We don't need to set this in multiline mode since multiline
             // matchers don't use optimizations related to line terminators.
-            // Moreover, a mulitline regex used with --null-data should
+            // Moreover, a multiline regex used with --null-data should
             // be allowed to match NUL bytes explicitly, which this would
             // otherwise forbid.
             if self.null_data {
@@ -517,7 +519,7 @@ impl HiArgs {
     /// When this returns false, it is impossible for ripgrep to ever report
     /// a match.
     pub(crate) fn matches_possible(&self) -> bool {
-        if self.patterns.patterns.is_empty() {
+        if self.patterns.patterns.is_empty() && !self.invert_match {
             return false;
         }
         if self.max_count == Some(0) {
@@ -562,7 +564,16 @@ impl HiArgs {
         wtr: W,
     ) -> Printer<W> {
         let summary_kind = if self.quiet {
-            SummaryKind::Quiet
+            match search_mode {
+                SearchMode::FilesWithMatches
+                | SearchMode::Count
+                | SearchMode::CountMatches
+                | SearchMode::JSON
+                | SearchMode::Standard => SummaryKind::QuietWithMatch,
+                SearchMode::FilesWithoutMatch => {
+                    SummaryKind::QuietWithoutMatch
+                }
+            }
         } else {
             match search_mode {
                 SearchMode::FilesWithMatches => SummaryKind::PathWithMatch,
@@ -570,10 +581,10 @@ impl HiArgs {
                 SearchMode::Count => SummaryKind::Count,
                 SearchMode::CountMatches => SummaryKind::CountMatches,
                 SearchMode::JSON => {
-                    return Printer::JSON(self.printer_json(wtr))
+                    return Printer::JSON(self.printer_json(wtr));
                 }
                 SearchMode::Standard => {
-                    return Printer::Standard(self.printer_standard(wtr))
+                    return Printer::Standard(self.printer_standard(wtr));
                 }
             }
         };
@@ -587,8 +598,8 @@ impl HiArgs {
     ) -> grep::printer::JSON<W> {
         grep::printer::JSONBuilder::new()
             .pretty(false)
-            .max_matches(self.max_count)
             .always_begin_end(false)
+            .replacement(self.replace.clone().map(|r| r.into()))
             .build(wtr)
     }
 
@@ -607,7 +618,6 @@ impl HiArgs {
             .hyperlink(self.hyperlink_config.clone())
             .max_columns_preview(self.max_columns_preview)
             .max_columns(self.max_columns)
-            .max_matches(self.max_count)
             .only_matching(self.only_matching)
             .path(self.with_filename)
             .path_terminator(self.path_terminator.clone())
@@ -647,7 +657,6 @@ impl HiArgs {
             .exclude_zero(!self.include_zero)
             .hyperlink(self.hyperlink_config.clone())
             .kind(kind)
-            .max_matches(self.max_count)
             .path(self.with_filename)
             .path_terminator(self.path_terminator.clone())
             .separator_field(b":".to_vec())
@@ -709,6 +718,7 @@ impl HiArgs {
         };
         let mut builder = grep::searcher::SearcherBuilder::new();
         builder
+            .max_matches(self.max_count)
             .line_terminator(line_term)
             .invert_match(self.invert_match)
             .line_number(self.line_number)
@@ -788,7 +798,7 @@ impl HiArgs {
                 attach_timestamps(haystacks, |md| md.created()).collect()
             }
         };
-        with_timestamps.sort_by(|(_, ref t1), (_, ref t2)| {
+        with_timestamps.sort_by(|(_, t1), (_, t2)| {
             let ordering = match (*t1, *t2) {
                 // Both have metadata, do the obvious thing.
                 (Some(t1), Some(t2)) => t1.cmp(&t2),
@@ -799,11 +809,7 @@ impl HiArgs {
                 // When both error, we can't distinguish, so treat as equal.
                 (None, None) => Ordering::Equal,
             };
-            if sort.reverse {
-                ordering.reverse()
-            } else {
-                ordering
-            }
+            if sort.reverse { ordering.reverse() } else { ordering }
         });
         Box::new(with_timestamps.into_iter().map(|(s, _)| s))
     }
@@ -893,7 +899,8 @@ impl HiArgs {
             .git_ignore(!self.no_ignore_vcs)
             .git_exclude(!self.no_ignore_vcs && !self.no_ignore_exclude)
             .require_git(!self.no_require_git)
-            .ignore_case_insensitive(self.ignore_file_case_insensitive);
+            .ignore_case_insensitive(self.ignore_file_case_insensitive)
+            .current_dir(&self.cwd);
         if !self.no_ignore_dot {
             builder.add_custom_ignore_filename(".rgignore");
         }
@@ -943,10 +950,12 @@ impl State {
     fn new() -> anyhow::Result<State> {
         use std::io::IsTerminal;
 
+        let cwd = current_dir()?;
+        log::debug!("read CWD from environment: {}", cwd.display());
         Ok(State {
             is_terminal_stdout: std::io::stdout().is_terminal(),
             stdin_consumed: false,
-            cwd: current_dir()?,
+            cwd,
         })
     }
 }
@@ -1080,9 +1089,18 @@ impl Paths {
             }
             paths.push(path);
         }
+        log::debug!("number of paths given to search: {}", paths.len());
         if !paths.is_empty() {
             let is_one_file = paths.len() == 1
-                && (paths[0] == Path::new("-") || paths[0].is_file());
+                // Note that we specifically use `!paths[0].is_dir()` here
+                // instead of `paths[0].is_file()`. Namely, the latter can
+                // return `false` even when the path is something resembling
+                // a file. So instead, we just consider the path a file as
+                // long as we know it isn't a directory.
+                //
+                // See: https://github.com/BurntSushi/ripgrep/issues/2736
+                && (paths[0] == Path::new("-") || !paths[0].is_dir());
+            log::debug!("is_one_file? {is_one_file:?}");
             return Ok(Paths { paths, has_implicit_path: false, is_one_file });
         }
         // N.B. is_readable_stdin is a heuristic! Part of the issue is that a
@@ -1169,7 +1187,7 @@ fn types(low: &LowArgs) -> anyhow::Result<ignore::types::Types> {
     let mut builder = ignore::types::TypesBuilder::new();
     builder.add_defaults();
     for tychange in low.type_changes.iter() {
-        match tychange {
+        match *tychange {
             TypeChange::Clear { ref name } => {
                 builder.clear(name);
             }
