@@ -1,5 +1,5 @@
 use crate::hay::SHERLOCK;
-use crate::util::{cmd_exists, sort_lines, Dir, TestCommand};
+use crate::util::{Dir, TestCommand, cmd_exists, sort_lines};
 
 // This file contains "miscellaneous" tests that were either written before
 // features were tracked more explicitly, or were simply written without
@@ -627,7 +627,7 @@ rgtest!(ignore_git_parent, |dir: Dir, mut cmd: TestCommand| {
 
     // Even though we search in foo/, which has no .gitignore, ripgrep will
     // traverse parent directories and respect the gitignore files found.
-    cmd.current_dir(dir.path().join("foo"));
+    cmd.current_dir("foo");
     cmd.assert_err();
 });
 
@@ -651,7 +651,7 @@ rgtest!(ignore_git_parent_stop, |dir: Dir, mut cmd: TestCommand| {
     dir.create_dir("foo/bar");
     dir.create("foo/bar/sherlock", SHERLOCK);
     cmd.arg("Sherlock");
-    cmd.current_dir(dir.path().join("foo").join("bar"));
+    cmd.current_dir("foo/bar");
 
     let expected = "\
 sherlock:For the Doctor Watsons of this world, as opposed to the Sherlock
@@ -682,7 +682,7 @@ rgtest!(ignore_git_parent_stop_file, |dir: Dir, mut cmd: TestCommand| {
     dir.create_dir("foo/bar");
     dir.create("foo/bar/sherlock", SHERLOCK);
     cmd.arg("Sherlock");
-    cmd.current_dir(dir.path().join("foo").join("bar"));
+    cmd.current_dir("foo/bar");
 
     let expected = "\
 sherlock:For the Doctor Watsons of this world, as opposed to the Sherlock
@@ -700,7 +700,7 @@ rgtest!(ignore_ripgrep_parent_no_stop, |dir: Dir, mut cmd: TestCommand| {
     dir.create_dir("foo/bar");
     dir.create("foo/bar/sherlock", SHERLOCK);
     cmd.arg("Sherlock");
-    cmd.current_dir(dir.path().join("foo").join("bar"));
+    cmd.current_dir("foo/bar");
 
     // The top-level .rgignore applies.
     cmd.assert_err();
@@ -733,7 +733,7 @@ rgtest!(no_parent_ignore_git, |dir: Dir, mut cmd: TestCommand| {
     dir.create("foo/sherlock", SHERLOCK);
     dir.create("foo/watson", SHERLOCK);
     cmd.arg("--no-ignore-parent").arg("Sherlock");
-    cmd.current_dir(dir.path().join("foo"));
+    cmd.current_dir("foo");
 
     let expected = "\
 sherlock:For the Doctor Watsons of this world, as opposed to the Sherlock
@@ -749,7 +749,7 @@ rgtest!(symlink_nofollow, |dir: Dir, mut cmd: TestCommand| {
     dir.create_dir("foo/baz");
     dir.create("foo/baz/sherlock", SHERLOCK);
     cmd.arg("Sherlock");
-    cmd.current_dir(dir.path().join("foo/bar"));
+    cmd.current_dir("foo/bar");
 
     cmd.assert_err();
 });
@@ -762,7 +762,7 @@ rgtest!(symlink_follow, |dir: Dir, mut cmd: TestCommand| {
     dir.create("foo/baz/sherlock", SHERLOCK);
     dir.link_dir("foo/baz", "foo/bar/baz");
     cmd.arg("-L").arg("Sherlock");
-    cmd.current_dir(dir.path().join("foo/bar"));
+    cmd.current_dir("foo/bar");
 
     let expected = "\
 baz/sherlock:For the Doctor Watsons of this world, as opposed to the Sherlock
@@ -922,6 +922,8 @@ be, to a very large extent, the result of luck. Sherlock Holmes
     eqnice!(expected, cmd.stdout());
 });
 
+// lz4 decompression tool doesn't work under RISC-V QEMU emulation in CI
+#[cfg(not(target_arch = "riscv64"))]
 rgtest!(compressed_lz4, |dir: Dir, mut cmd: TestCommand| {
     if !cmd_exists("lz4") {
         return;
@@ -952,6 +954,8 @@ be, to a very large extent, the result of luck. Sherlock Holmes
     eqnice!(expected, cmd.stdout());
 });
 
+// brotli decompression tool doesn't work under RISC-V QEMU emulation in CI
+#[cfg(not(target_arch = "riscv64"))]
 rgtest!(compressed_brotli, |dir: Dir, mut cmd: TestCommand| {
     if !cmd_exists("brotli") {
         return;
@@ -967,6 +971,8 @@ be, to a very large extent, the result of luck. Sherlock Holmes
     eqnice!(expected, cmd.stdout());
 });
 
+// zstd decompression tool doesn't work under RISC-V QEMU emulation in CI
+#[cfg(not(target_arch = "riscv64"))]
 rgtest!(compressed_zstd, |dir: Dir, mut cmd: TestCommand| {
     if !cmd_exists("zstd") {
         return;
@@ -1092,16 +1098,22 @@ rgtest!(type_list, |_: Dir, mut cmd: TestCommand| {
 // This order is important when sorting them by system time-stamps.
 fn sort_setup(dir: Dir) {
     use std::{thread::sleep, time::Duration};
-
+    // As reported in https://github.com/BurntSushi/ripgrep/issues/3071
+    // this test fails if sufficient delay is not given on Windows/Aarch64.
+    let delay = if cfg!(all(windows, target_arch = "aarch64")) {
+        Duration::from_millis(1000)
+    } else {
+        Duration::from_millis(100)
+    };
     let sub_dir = dir.path().join("dir");
     dir.create("a", "test");
-    sleep(Duration::from_millis(100));
+    sleep(delay);
     dir.create_dir(&sub_dir);
-    sleep(Duration::from_millis(100));
+    sleep(delay);
     dir.create(sub_dir.join("c"), "test");
-    sleep(Duration::from_millis(100));
+    sleep(delay);
     dir.create("b", "test");
-    sleep(Duration::from_millis(100));
+    sleep(delay);
     dir.create(sub_dir.join("d"), "test");
 }
 
